@@ -14,11 +14,8 @@ import asyncio
 import logging
 import time
 import rapidjson
-import aiohttp
 from .config import Config
 
-LOG = logging.getLogger(__name__)
-
 
 class VerificationError(Exception):
     pass
@@ -38,17 +35,20 @@ class Verifier:
         self.running = True
         self.queue = asyncio.Queue()
         # nip05_verification can be "enabled", "disabled", or "passive"
-        self.enabled = Config.nip05_verification == 'enabled'
+        self.is_enabled = Config.nip05_verification == 'enabled'
+        self.should_verify = Config.nip05_verification in ('enabled', 'passive')
+        if self.should_verify:
+            self.log = logging.getLogger(__name__)
 
     async def update_metadata(self, cursor, event):
         # metadata events are evaluated as candidates
         try:
             meta = rapidjson.loads(event.content)
         except Exception:
-            LOG.exception("bad metadata")
+            self.log.exception("bad metadata")
         else:
             identifier = meta.get('nip05', '')
-            LOG.debug("Found identifier %s in event %s", identifier, event)
+            self.log.debug("Found identifier %s in event %s", identifier, event)
             if '@' in identifier:
                 # queue this identifier as a candidate
                 domain = identifier.split('@', 1)[1].lower()
@@ -56,7 +56,7 @@ class Verifier:
                     await self.queue.put([None, identifier, 0, event.pubkey, event.id_bytes])
                     return True
                 else:
-                    LOG.error("Illegal domain in identifier %s", identifier)
+                    self.log.error("Illegal domain in identifier %s", identifier)
         return False
 
     def check_allowed_domains(self, domain):
@@ -80,77 +80,78 @@ class Verifier:
         if event.kind == 0:
             is_candidate = await self.update_metadata(cursor, event)
             if not is_candidate:
-                if self.enabled:
+                if self.is_enabled:
                     raise VerificationError("rejected: metadata must have nip05 tag")
                 else:
-                    LOG.warning("Attempt to save metadata event %s from %s without nip05 tag", event.id, event.pubkey)
+                    self.log.warning("Attempt to save metadata event %s from %s without nip05 tag", event.id, event.pubkey)
             else:
                 return True
 
         await cursor.execute(self.VERIFICATION_QUERY, (event.pubkey, ))
         row = await cursor.fetchone()
         if not row:
-            if self.enabled:
+            if self.is_enabled:
                 raise VerificationError(f"rejected: pubkey {event.pubkey} must be verified")
             else:
-                LOG.warning('pubkey %s is not verified.', event.pubkey)
+                self.log.warning('pubkey %s is not verified.', event.pubkey)
         else:
             vid, identifier, verified_at, failed_at, created_at = row
-            LOG.debug("Checking verification for %s verified:%s created:%s", identifier, verified_at, created_at)
+            self.log.debug("Checking verification for %s verified:%s created:%s", identifier, verified_at, created_at)
             now = time.time()
             if ((verified_at or 0) + Config.verification_expiration) < now:
                 # verification has expired
-                if self.enabled:
+                if self.is_enabled:
                     raise VerificationError(f"rejected: verification expired for {identifier}")
                 else:
-                    LOG.warning("verification expired for %s on %s", identifier, verified_at)
+                    self.log.warning("verification expired for %s on %s", identifier, verified_at)
 
             uname, domain = identifier.split('@', 1)
             domain = domain.lower()
             if not self.check_allowed_domains(domain):
-                if self.enabled:
+                if self.is_enabled:
                     raise VerificationError(f"rejected: {domain} not allowed")
                 else:
-                    LOG.warning("verification for %s not allowed", identifier)
-        LOG.debug("Verified %s", event.pubkey)
+                    self.log.warning("verification for %s not allowed", identifier)
+        self.log.debug("Verified %s", event.pubkey)
         return True
 
     async def verification_task(self, db):
-        LOG.info("Starting verification task. Interval %s", Config.verification_update_frequency)
+        self.log.info("Starting verification task. Interval %s", Config.verification_update_frequency)
         last_run = 0
+        query = f"""
+            SELECT v.id, identifier, verified_at, pubkey, metadata_id FROM verification as v
+            LEFT JOIN event ON v.metadata_id = event.id
+            WHERE pubkey IS NOT NULL AND
+            (? - verified_at > {Config.verification_expiration})
+            ORDER BY verified_at DESC
+        """
         while self.running:
             candidate = await self.queue.get()
             if candidate is None:
                 break
-            LOG.debug("Got candidate %s", candidate)
+            self.log.debug("Got candidate %s", candidate)
             candidates = []
             try:
                 if (time.time() - last_run) > Config.verification_update_frequency:
-                    LOG.debug("running batch query")
+                    self.log.debug("running batch query")
                     async with db.cursor() as cursor:
                         try:
-                            await cursor.execute(f"""
-                                SELECT v.id, identifier, verified_at, pubkey, metadata_id FROM verification as v
-                                LEFT JOIN event ON v.metadata_id = event.id
-                                WHERE pubkey IS NOT NULL AND
-                                (? - verified_at > {Config.verification_expiration})
-                                ORDER BY verified_at DESC
-                            """, (int(time.time()), ))
+                            await cursor.execute(query, (int(time.time()), ))
                         except Exception:
-                            LOG.exception('batch query')
+                            self.log.exception('batch query')
                             continue
                         async for row in cursor:
                             candidates.append(row)
                             # vid, identifier, verified_at, pubkey = row
             except Exception:
-                LOG.exception("batch_query")
+                self.log.exception("batch_query")
                 continue
             candidates.append(candidate)
 
             try:
                 success, failure = await self.process_verifications(candidates)
             except Exception:
-                LOG.exception("process_verifications")
+                self.log.exception("process_verifications")
             else:
                 if success or failure:
                     async with db.cursor() as cursor:
@@ -167,14 +168,15 @@ class Verifier:
                             else:
                                 await cursor.execute(self.FAILURE_QUERY, (vid, ))
                         await db.commit()
-                    LOG.info("Saved success:%d failure:%d", len(success), len(failure))
+                    self.log.info("Saved success:%d failure:%d", len(success), len(failure))
             last_run = time.time()
 
-        LOG.info("Stopped verification task")
+        self.log.info("Stopped verification task")
 
     async def process_verifications(self, candidates):
         success = []
         failure = []
+        import aiohttp
         async with aiohttp.ClientSession(timeout=aiohttp.ClientTimeout(total=10.0), json_serialize=rapidjson.dumps) as session:
 
             for vid, identifier, verified_at, pubkey, metadata_id in candidates:
@@ -183,35 +185,35 @@ class Verifier:
                 domain = domain.lower()
                 if not self.check_allowed_domains(domain):
                     # how did this record get here?
-                    LOG.warning("skipping verification for disallowed domain %s", identifier)
+                    self.log.warning("skipping verification for disallowed domain %s", identifier)
                     continue
                 # request well-known url
                 url = f'https://{domain}/.well-known/nostr.json?name={uname}'
-                LOG.info("Requesting %s", url)
+                self.log.info("Requesting %s", url)
 
                 try:
                     async with session.get(url) as response:
                         data = await response.json(loads=rapidjson.loads)
                     names = data['names']
                 except Exception:
-                    LOG.exception("Failure verifying %s from %s", identifier, url)
+                    self.log.exception("Failure verifying %s from %s", identifier, url)
                     failure.append([vid, identifier, metadata_id])
                 else:
                     if names.get(uname, '') != pubkey:
-                        LOG.warning("Could not verify %s=%s from %s", identifier, pubkey, url)
+                        self.log.warning("Could not verify %s=%s from %s", identifier, pubkey, url)
                         failure.append([vid,  identifier, metadata_id])
                     else:
-                        LOG.info("Verified %s=%s from %s", identifier, pubkey, url)
+                        self.log.info("Verified %s=%s from %s", identifier, pubkey, url)
                         success.append([vid, identifier, metadata_id])
 
         return success, failure
 
     async def start(self, db):
-        if Config.nip05_verification in ('enabled', 'passive'):
+        if self.should_verify:
             asyncio.create_task(self.verification_task(db))
 
     async def stop(self):
-        if Config.nip05_verification in ('enabled', 'passive'):
+        if self.should_verify:
             self.running = False
             await self.queue.put(None)
 
